const matchGraphQL = { urls: ["https://graphql.landsbankinn.is/v2"] };

import { fetchAccounts } from '../landsbankinn';
import { PartialStateUpdate } from '../state';
import { State } from '../state/level';

type HttpHeaders = browser.webRequest.HttpHeaders;

function extractAuthStateFromHeaders(headers: HttpHeaders): PartialStateUpdate {
    let apiKey = '';
    let authToken = '';
    let fingerprintValue = '';

    for (const header of headers) {
        switch (header.name.toLowerCase()) {
            case "authorization":
                authToken = header.value || '';
                break;
            case "apikey":
                apiKey = header.value || '';
                break;
            case "fingerprintvalue":
                fingerprintValue = header.value || '';
                break;
        }
    }

    return { apiKey, authToken, fingerprintValue };
}

browser.webRequest.onBeforeSendHeaders.addListener(e => {
    // TODO if request errors, then clear state and try again.
    (async function() {
        let state = await State.current;
        if (!state.ready && e.requestHeaders) {
            console.info('Will update auth info from headers');

            // Lock ready to true so we don't have multiple attempts
            // at getting account info.
            await State.update({ ready: true });
            let newState = extractAuthStateFromHeaders(e.requestHeaders);
            state = await State.update(newState);

            if (!newState.accounts) {
                const accounts = await fetchAccounts(state);
                await State.update({ accounts });
                console.info('Acquired account data');
            }
        }
    })();
}, matchGraphQL, ["requestHeaders"]);

export { }

// Here we intercept and record the necessary auth info. But what if
// we cannot do that, if user logs directly into a page where there
// are no graphql calls?

// We can get device fingerprint via content_script, and call session.

// Have a message called ENSURE_STATE.
// Sent to background page w/ device_fingerprint.
// If state not ready, call session endpoint, and parse output.
// Record state.
// Content script for ensure state runs everywhere.
// There is a value on the page that has everything we need!
