import { isStatePopulated, isStateClear, PartialStateUpdate, ExporterState, isStateAbleToRequest } from "../state";
import { Status } from '../state';
import { LevelState } from '../state/level';
import * as landsbankinn from '../landsbankinn';
import { fetchAccounts } from "../landsbankinn";
import { Actions, EnsureStateMessage, EnsureStateReply, GetAccountsReply } from './messages';

// State singleton (move to its own file if we eventually get more
// than 1 background script)
const State = new LevelState();

async function ensureDataPopulated(state: ExporterState) {
    if (isStateAbleToRequest(state) && !isStatePopulated(state)) {
        const accounts = await fetchAccounts(state);
        await State.update({ status: Status.Populated, accounts });
        console.info('Populated account data');
    }
}

async function ensureState(message: EnsureStateMessage): Promise<EnsureStateReply> {
    console.info('Ensuring state');
    const clientState: PartialStateUpdate = message.clientState;;
    let state = await State.current;

    if (isStateClear(state)) {
        await State.update({ status: Status.Updating });
        const session = await landsbankinn.authSession();
        await State.update({
            status: Status.AbleToRequest, authToken: session.accessToken, ...clientState
        });

        console.info('Updated auth state from client and auth session');
    }

    state = await State.current;
    await ensureDataPopulated(state);
    return { state: 'Populated' };
}

async function getAccounts(): Promise<GetAccountsReply> {
    const state = await State.current;
    let message: any;

    await ensureDataPopulated(state);

    // Should always be populated, but we can be careful anyway.
    if (isStatePopulated(state)) {
        message = {
            accounts: (await State.current).accounts,
            error: null
        };
    }
    else {
        message = {
            accounts: undefined,
            error: new Error('state not yet populated')
        };
    }

    return message;
}

browser.runtime.onMessage.addListener(async (message) => {
    switch (message.action) {
        case Actions.EnsureState:
            return await ensureState(message);
        case Actions.GetAccounts:
            return await getAccounts();
    }
});

export { }
