import React, { FormEventHandler } from 'jsx-dom'; // necessary for react to work
import { Account } from '../../landsbankinn/models';
import $ from 'cash-dom';

interface AccountListProps {
    accounts: Array<Account>;
}

const formatAccountNumber = (accountNumber: string) =>
    accountNumber.substring(0, 4) + '-' +
    accountNumber.substring(4, 6) + '-' +
    accountNumber.substring(6, 12);

//getValues extracts the account numbers and date range for
//transactions to download. Basically the "submit button."
//TODO why not use an actual form...
const getValues = (exportDiv: HTMLElement): Array<string> =>
    //@ts-expect-error
    $(`#${exportDiv.id} input[type="checkbox"]`)
        .filter('[data-checkbox-type="account"]:checked')
        //@ts-expect-error
        .map((_, el) => (el as HTMLInputElement).name).get();

const SelectAllRow = () => {
    const boxStyle = { display: 'inline-block', width: "100%", cursor: 'pointer' };

    const selectAllHandler = (e: MouseEvent) => {
        const currentlyChecked = $('#check-all').prop('checked');

        //@ts-expect-error
        if ($(e.currentTarget).is($('#check-all')) == false) {
            //The checkbox can toggle itself, by itself. Only toggle
            //from other clicks
            $(`#check-all`).prop('checked', !currentlyChecked);
        }

        const checked = $('#check-all').prop('checked');
        $('#account-export-list input[type="checkbox"]').prop('checked', checked);
    };

    return (
        <tr>
            <td>
                <input id="check-all" type="checkbox" data-checkbox-type="select-all"
                    onClick={selectAllHandler} />
            </td>
            <td onClick={selectAllHandler}>
                <span style={boxStyle}>Select All</span>
            </td>
            <td onClick={selectAllHandler}>
                <span style={boxStyle}>
                    N/A (Selects all accounts below)
                </span>
            </td>
        </tr>
    );
}

const AccountRow = (props: { accountNumber: string; name: string; }) => {
    const boxStyle = { display: 'inline-block', width: "100%", cursor: 'pointer' };

    const selectAccountHandler = (accountNumber: string) => () =>
        $(`#export-${accountNumber}`)
            .prop('checked', !$(`#export-${accountNumber}`).prop('checked'));

    return (
        <tr>
            <td>
                <input id={`export-${props.accountNumber}`}
                    data-checkbox-type="account"
                    type="checkbox" name={props.accountNumber} />
            </td>
            <td onClick={selectAccountHandler(props.accountNumber)}>
                <span style={boxStyle}>
                    {formatAccountNumber(props.accountNumber)}
                </span>
            </td>
            <td onClick={selectAccountHandler(props.accountNumber)}>
                <span style={boxStyle}>
                    {props.name}
                </span>
            </td>
        </tr>
    );
}

const AccountList = (props: AccountListProps) => {
    const accountList = props.accounts.map(acct =>
        <AccountRow accountNumber={acct.accountNumber} name={acct.name} />
    );

    return (
        <div id="account-export-list">
            <table>
                <thead>
                    <tr>
                        <th scope="colgroup">Select</th>
                        <th scope="colgroup">Account Number</th>
                        <th scope="colgroup">Account Name</th>
                    </tr>
                </thead>
                <tbody>
                    <SelectAllRow />
                    {accountList}
                </tbody>
            </table>
        </div >
    );
}

const datePickerID = (label: string) => `export-date-${label}`;

const DatePicker = (props: { label: string; }) => (
    <div style={{ display: 'inline-block' }}>
        <label htmlFor={datePickerID(props.label)}>{props.label}</label>
        <input id={datePickerID(props.label)} type="date" />
    </div>
);

const DownloadButton = () => {
    return (
        <div style={{ display: 'inline-block' }}>
            <button>Download Transactions</button>
        </div>
    );
}

interface ExportDivProps {
    fromLabel: string;
    toLabel: string;
    accounts: Account[];
    onDownload: (accountNumbers: string[]) => void;
}

export const ExportDiv = (props: ExportDivProps) => {
    const submitHandler = (e: Event) => {
        e.preventDefault();
        const accountNumbers = getValues($('#account-export').get(0)!);
        props.onDownload(accountNumbers);
    }

    return (
        <div id="account-export" class="content-box fill shadow ui-form" >
            <h2>Download Transaction Statements</h2>
            <p>
                Here, you can export statements from one or more of your accounts
                over a specified date range.
            </p>

            <form id="export-transaction-form" onSubmit={submitHandler}>
                <div id="export-date-pickers">
                    <DatePicker label={props.fromLabel} />
                    <DatePicker label={props.toLabel} />
                    <DownloadButton />
                </div>

                <div class="row-column">
                    <div class="column-span2 column-span2--base">
                        <AccountList accounts={props.accounts} />
                    </div>
                </div>
            </form>
        </div>
    );
}
