import { Account } from '../landsbankinn/models';

export enum Status {
    NotReady,  // completely clear
    Updating,  // in process of gathering auth data
    AbleToRequest,     // able to make requests
    Populated  // has all necessary information
}

export interface ExporterState {
    apiKey: string;
    authToken: string;
    fingerprintValue: string;
    accounts: Array<Account>;
    status: Status
}

export interface PartialStateUpdate {
    apiKey?: string;
    authToken?: string;
    fingerprintValue?: string;
    accounts?: Array<Account>;
    status?: Status
}

// TODO export it and replace direct references to level with it
interface StateStore {
    readonly current: Promise<ExporterState>;
    update(u: ExporterState | PartialStateUpdate): Promise<ExporterState>;
    clear(): Promise<any>;

}

export const isStateClear = (state: ExporterState) =>
    state.status === undefined || state.status == Status.NotReady;

export const isStatePopulated = (state: ExporterState) =>
    state.status == Status.Populated;

export const isStateAbleToRequest = (state: ExporterState) =>
    state.status == Status.AbleToRequest || state.status == Status.Populated;
